/*
 * $Id: main.cpp,v 1.10 2006/03/19 17:09:14 tsuruoka Exp $
 */

#include <stdio.h>
#include <fstream>
#include <map>
#include <list>
#include <iostream>
#include <sstream>
#include "maxent.h"
#include "common.h"

using namespace std;

bool IGNORE_DISTINCTION_BETWEEN_NNP_AND_NN = false;

const int NUM_TRAIN_SENTENCES = 99999999;
//const int NUM_TRAIN_SENTENCES = 10000;
//const int NUM_TRAIN_SENTENCES = 100;

const int NUM_TEST_SENTENCES = 99999999;

//ME_Sample mesample(const vector<Token> &vt, int i, const string & prepos, const string & prepos2);
ME_Sample mesample(const vector<Token> &vt, int i, const string & prepos);
void viterbi(vector<Token> & vt, const ME_Model & me);
string postag(const string & s, const ME_Model & me);
string bidir_postag(const string & s, const vector<ME_Model> & vme, bool dont_tokenize);

void
bidir_postagging(vector<Sentence> & vs,
                 const multimap<string, string> & tagdic,
                 const vector<ME_Model> & vme);

int
bidir_train(const vector<Sentence> & vs, int para, int nservers);

vector<string> split(const string & s, char d)
{
  vector<string> tmp;
  
  int begin = 0;
  int x = 0;
  while (1) {
    if (s[x] == d || x == s.size()) {
      tmp.push_back(s.substr(begin, x - begin));
      begin = x + 1;
    }
    if (x == s.size()) break;
    x++;
  }
  return tmp;
}

void evaluate(const vector<Sentence> & vs)
{
  int ncorrect = 0;
  int ntotal = 0;
  int npreds = 0;
  int nambs = 0;
  for (vector<Sentence>::const_iterator i = vs.begin(); i != vs.end(); i++) {
    const Sentence & s = *i;
    for (int j = 0; j < s.size(); j++) {
      ntotal++;
      string pos = s[j].pos;
      string prd = s[j].prd;
      if (IGNORE_DISTINCTION_BETWEEN_NNP_AND_NN) {      
        if (pos == "NNP") pos = "NN";
        if (pos == "NNPS") pos = "NNS";
        if (prd == "NNP") prd = "NN";
        if (prd == "NNPS") prd = "NNS";
      }
      //      if (pos == prd) ncorrect++;
      vector<string> prds = split(prd, '|');
      npreds += prds.size();
      if (prds.size() > 1) nambs++;
      if (find(prds.begin(), prds.end(), pos) != prds.end()) ncorrect++;
      cout << s[j].str << "\t" << s[j].pos << "\t" << s[j].prd << endl;
    }
  }
  fprintf(stderr, "%d / %d = %f\n", ncorrect, ntotal, (double)ncorrect / ntotal);
  //  fprintf(stderr, "avg. # tags/token = %f\n", npreds / (double)ntotal);
  //  fprintf(stderr, "%% amb tokens = %f\n", 100.0 * nambs / (double)ntotal);
}

void read_pos(const string & filename, vector<Sentence> & vs, int num_sentences = -1)
{
  static ParenConverter paren_converter;
 
  ifstream ifile(filename.c_str());

  string line;
  int n = 0;
  cerr << "reading " << filename;
  while (getline(ifile,line)) {
    istringstream is(line);
    string s;
    Sentence sentence;
    while (is >> s) {
      string::size_type i = s.find_last_of('/');
      string str = s.substr(0, i);
      string pos = s.substr(i+1);
      str = paren_converter.Ptb2Pos(str);
      pos = paren_converter.Pos2Ptb(pos);
      //      cout << str << "\t" << pos << endl;
      Token t(str, pos);
      sentence.push_back(t);
    }
    vs.push_back(sentence);
    if (vs.size() >= num_sentences) break;
    if (n++ % 10000 == 0) cerr << ".";
  }
  cerr << endl;

  ifile.close();
}


int main(int argc, char** argv)
{
  bool dont_tokenize = false;
  
  istream *is(&std::cin);
  ostream *os(&std::cout);

  int para = -1, nservers = -1;
  string ifilename, ofilename;
  if (argc == 3) {
    para = atoi(argv[1]);
    nservers = atoi(argv[2]);
  }
  for (int i = 1; i < argc; i++) {
    string v = argv[i];
    if (v == "-i" && i < argc-1) {
      ifilename = argv[i+1];
    }
    if (v == "-o" && i < argc-1) {
      ofilename = argv[i+1];
    }
    if (v == "-nt") { dont_tokenize = true; continue; }
    if (v == "--train") {
      vector<Sentence> trains;
      read_pos("./train.pos", trains, NUM_TRAIN_SENTENCES);
      bidir_train(trains, para, nservers);
      exit(0);
    }
    if (v == "--test") {
      vector<ME_Model> vme(16);
      for (int i = 0; i < 16; i++) {
        char buf[1000];
        sprintf(buf, "./model.bidir.%d", i);
        cerr << "loading " << string(buf) << endl;
        vme[i].load_from_file(buf);
      }

      vector<Sentence> tests;
      cerr << "evaluation on wsj" << endl;
      read_pos("./test.pos", tests, NUM_TEST_SENTENCES);
      multimap<string, string> tagdic;
      bidir_postagging(tests, tagdic, vme);
      evaluate(tests);
      exit(0);
    }
  }
  ifstream ifile;
  if (ifilename != "" && ifilename != "-") {
    ifile.open(ifilename.c_str());
    if (!ifile) { cerr << "error: cannot open " << ifilename << endl; exit(1); }
    is = &ifile;
  }
  ofstream ofile;
  if (ofilename != "" && ofilename != "-") {
    ofile.open(ofilename.c_str());
    if (!ofile) { cerr << "error: cannot open " << ofilename << endl; exit(1); }
    os = &ofile;
  }
  
  vector<ME_Model> vme(16);
  for (int i = 0; i < 16; i++) {
    char buf[1000];
    sprintf(buf, "./model.bidir.%d", i);
    cerr << "loading " << string(buf) << endl;
    vme[i].load_from_file(buf);
  }

  string line;
  while (getline(*is, line)) {
    string postagged = bidir_postag(line, vme, dont_tokenize);
    (*os) << postagged << endl;
  }
  exit(0);


  
}

/*
 * $Log: main.cpp,v $
 * Revision 1.10  2006/03/19 17:09:14  tsuruoka
 * add output_multiple_hypotheses
 *
 * Revision 1.9  2005/09/01 14:14:51  tsuruoka
 * add ignore_distinction_between_nnp_and_nn
 *
 * Revision 1.8  2005/02/16 10:45:39  tsuruoka
 * acl05 submit
 *
 * Revision 1.7  2005/01/10 13:44:36  tsuruoka
 * a
 *
 * Revision 1.6  2004/12/30 10:34:56  tsuruoka
 * add bidir_decode_search
 *
 * Revision 1.5  2004/12/25 10:47:12  tsuruoka
 * add load_tag_dictionary()
 *
 * Revision 1.4  2004/12/25 09:22:57  tsuruoka
 * add make_tag_dictionary()
 *
 * Revision 1.3  2004/12/21 13:54:46  tsuruoka
 * add bidir.cpp
 *
 * Revision 1.2  2004/12/20 12:06:24  tsuruoka
 * change the data
 *
 * Revision 1.1  2004/07/16 13:40:42  tsuruoka
 * init
 *
 */

